use std::fmt::{Display, Debug};

pub trait Summary {
    fn summary(&self) -> String;
}

pub struct BlogPost {
    pub title: String,
    pub author: String,
    pub content: String,
}

impl Summary for BlogPost {
    fn summary(&self) -> String {
        format!("{} by {}", self.title, self.author)
    }
}

// 简单写法
//pub fn summary1(i: &impl Summary) {
pub fn summary1(i: impl Summary) {
    println!("{}", i.summary())
}

// trait bound 语法
//pub fn summary2<T: Summary>(i: &T) {
pub fn summary2<T: Summary>(i: T) {
    println!("{}", i.summary())
}

// 实现多个 trait
pub fn summary3(i: impl Summary + Display) {
    println!("{}", i.summary())
}

pub fn summary4<T: Summary + Display>(i: T) {
    println!("{}", i.summary())
}

// where 子句，类似于 c# 的写法
pub fn summary5<T, U>(x: T, y: U) -> String
where
    T: Summary + Display,
    U: Debug,
{
    format!("{}", x.summary())
}

fn main() {
    let p = BlogPost {
        title: String::from("some title"),
        author: String::from("xiexianbin"),
        content: String::from("abcd23"),
    };

    // summary1(p);
    summary2(p);
}
